/*
File: AppletBase.java
Date		Author		Changes
Sep 09 96	Lukasz Beca	Created
11  13 97   K Olszewski changed application start

*/


package webwisdom.tango.interfaces;

import java.applet.*;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.main.LocalBase;
import webwisdom.tango.structures.AppletHandle;
import netscape.javascript.JSObject;
import webwisdom.tango.messages.CAppRequestEventMessage;


/**
 * This class should be extended by applet works with our communication system
 * 
 *@author Lukasz Beca
 *@see Interfaces in CIV Communication System documentation
 **/

public abstract class AppletBase extends Applet implements AppletBaseInterf {

  private AppletHandle h;
  private boolean finished;
  private boolean inwindow;
  private int AID=0;

  //interface for applet
  //public void register();
  //public void send(AppEventMessage m);
  //public void send(AppEventMessage m, int userlist[]);
  //public void destroy();

  /**
  * Registers an applet in the communication system. Messages can be send and received by the applet only after this method is called so it should be placed in init method.  
  **/

  public void register(){

    LocalBase p;
    JSObject window, parent;
    String name;
    int AIDend, SIDend;
    int SID, AT;
    byte[] tab = new byte[1];
    tab[0] = 1; // question if applet is a master or not
    CAppRequestEventMessage mes = new CAppRequestEventMessage(tab);


    try {   //added support for AID
      
	  	 window = (JSObject) JSObject.getWindow(this);
		 parent = (JSObject) window.getMember("top");  		 
 		 name = (String) parent.getMember("name");	
	
 	   	System.out.println("AppletBase: window : " + name);
			
	    AIDend = name.indexOf('_');
	    SIDend = name.indexOf('_', AIDend + 1);
			 AID = Integer.parseInt(name.substring(3, AIDend));
			 SID = Integer.parseInt(name.substring(AIDend + 1, SIDend));
			 AT = Integer.parseInt(name.substring(SIDend + 1));
			 
			 System.out.println("AppletBase: AID " + AID + " SID " + SID + " AT " + AT);
	  
	  }
    catch (Exception e) {
        System.out.println("AID not found");     
    				e.printStackTrace();
    	return;
	}

    finished = false;
    p = getPlugin();
    if (p == null) {
      System.out.println("AppletBase: register failed, switching to standalone mode");     
      h = null;
      return;
    }
   
    h = p.reg.registerApplet(this, AID, SID, AT);
    send(mes);
         
  }

  /**
  * Sends message to other applets in session or to the control application (it depends on the type of message) 
  *@param m message
  *@see AppEventMessage
  **/

  public void send(AppEventMessage m) {

    if (h != null) 
      h.sendAppToOther(m);
    
  }
  
  public void send(int userlist[], AppEventMessage m) {
  
  		if (h != null) 
			h.sendAppToOther(userlist, m);
  
  }

  public void destroy() {

    System.out.println("AppletBase: destroy called");
   
    if (h != null) {
      h.remove();
      h = null;
    }
    super.destroy();
  }

  //interface for AppletHandle
  //public abstract synchronized void receive(AppEventMessage m);
  //public JSObject getWindow();
  //public void finish();
  //public boolean inWindow();

  /**
  * This method must be implemented by applet that extends class AppletBase. If the message to the applet arrives this method is called by our system with message as a parameter.
  *@param m message
  *@see AppEventMessage
  **/

  public abstract synchronized void receive(AppEventMessage m);

  public JSObject getJSWindow() {
  
    JSObject window;

    window =  (JSObject) JSObject.getWindow(this);

    if (window == null) {
      System.out.println("AppletBase: unable to get reference to window");	
      window = null;
    } 
    
    return window;    
  }

  public boolean inWindow() {

    return inwindow;
  }

  //internal methods
  //private LocalBase getPlugin();

  private LocalBase getPlugin() {

    JSObject tmp, parent, frame, doc, opener; 
    Object test;
    
    tmp = (JSObject) JSObject.getWindow(this);

    if (tmp == null) {
      System.out.println("AppletBase: unable to get reference to window");	
      return null;
    }
    parent = (JSObject) tmp.getMember("top");
    if (parent == null) {
      System.out.println("AppletBase: unable to get reference to parent");
      return null;
    }
    
    // test for frame
    
    test = parent.getMember("navbar");

    if (!(test instanceof JSObject)) {
      System.out.println("AppletBase: frame not detected, switching to window mode");
      inwindow = true;

      opener = (JSObject) parent.getMember("opener");
      if (opener == null) {
	System.out.println("AppletBase: unable to get reference to opener");
	return null;
      }

      frame = (JSObject) opener.getMember("navbar");
      if (frame == null) {
	System.out.println("AppletBase: unable to get reference to frame");
	return null;
      }
   
      doc = (JSObject) frame.getMember("document");
      if (doc == null) {
	System.out.println("AppletBase: unable to get reference to document");
	return null;
      }
   
      return (LocalBase) doc.getMember("local");
    
    } else {
      
      inwindow = false;
      frame = (JSObject) test;

      doc = (JSObject) frame.getMember("document");
      if (doc == null) {
      System.out.println("AppletBase: unable to get reference to document");
      return null;
      }
   
      return (LocalBase) doc.getMember("local");

    }   
  }      
}












